unit KA.Data.KADao.FindButton;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.Forms,
  Borland.Vcl.Dialogs,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls;

type
  TKADaoFindButton = class(TButton)
  private
    { Private declarations }
    FDataLink     : TFieldDataLink;
    FBefore       : Boolean;
    FLastFilter   : TStringList;
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   HandleClickEvent;
    Procedure   DataChange(Sender: TObject);
    Procedure   ActiveChange(Sender: TObject);
    procedure   Loaded; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Procedure   Click; override;
  published
    { Published declarations }
    Property    DataSource  : TDataSource Read GetDataSource Write SetDataSource;
    Property    CallBefore  : Boolean     Read FBefore         Write FBefore;
  end;

procedure Register;

implementation
Uses
  KA.Data.KADao.Table,
  KA.Data.KADao.FindSeekUnit;

Const
 dbOpenTable = 1;

Constructor TKADaoFindButton.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  Caption     := 'Find';
  FLastFilter:= TStringList.Create;
  FDataLink  := TFieldDataLink.Create;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnActiveChange  := ActiveChange;
  FBefore    := True;
End;

Destructor TKADaoFindButton.Destroy;
Begin
  FLastFilter.Free;
  FDataLink.Free;
  FDataLink := nil;
  Inherited Destroy;
End;

Function TKADaoFindButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoFindButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
end;

Procedure TKADaoFindButton.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoFindButton.DataChange(Sender: TObject);
Begin
if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoFindButton.Loaded;
Begin
  Inherited Loaded;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType<>dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

procedure TKADaoFindButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource := nil;
end;

Procedure TKADaoFindButton.HandleClickEvent;
Var
 BK : TBookmarkStr;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
       if TKaDaoTable(DataSource.Dataset).Bookmarkable Then
          BK := TKaDaoTable(DataSource.Dataset).Bookmark;
       Application.CreateForm(TKADaoFindSeekDialog, KADaoFindSeekDialog);
       if KADaoFindSeekDialog.Execute(TKaDaoTable(DataSource.DataSet), False) Then
          Begin
          End
       Else
          Begin
           if TKaDaoTable(DataSource.Dataset).Bookmarkable Then
              TKaDaoTable(DataSource.Dataset).Bookmark:=BK;
          End;
       KADaoFindSeekDialog.Free;
     End;
End;

Procedure TKADaoFindButton.Click;
Begin
 if FBefore Then inherited Click;
 HandleClickEvent;
 if NOT FBefore Then inherited Click;
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoFindButton]);
end;

end.
