unit KA.Data.KADao.IndexesListBox;
interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.StdCtrls,
  KA.Data.KADao.Table;

type
  TKADaoIndexesListBox = class(TListBox)
  private
    { Private declarations }
    FDataLink       : TFieldDataLink;
    FInGetting      : Boolean;
    FAutoOpen       : Boolean;
    Procedure          ActiveChange(Sender: TObject);
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   SetAutoOpen(Value: Boolean);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   Loaded; override;
    Procedure   Click; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
  published
    { Published declarations }
    Property    AutoOpen          : Boolean     Read FAutoOpen        Write SetAutoOpen;
    Property    DataSource        : TDataSource  Read GetDataSource Write SetDataSource;
  end;

procedure Register;

implementation
Const
   dbOpenTable = 1;

Constructor TKADaoIndexesListBox.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FAutoOpen                 := True;
  FInGetting                := False;
  FDataLink                 := TFieldDataLink.Create;
  FDataLink.Control         := Self;
  FDataLink.OnActiveChange  := ActiveChange;
End;

Destructor TKADaoIndexesListBox.Destroy;
Begin
 FDataLink.Free;
 FDataLink := nil;
 inherited Destroy;
End;

procedure TKADaoIndexesListBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then
      Begin
        DataSource := nil;
      End;
end;


Procedure TKADaoIndexesListBox.Loaded;
Begin
 Inherited Loaded;
 Items.Clear;
End;

Procedure TKADaoIndexesListBox.Click;
Begin
  Inherited Click;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.Dataset.Active)
  And (DataSource.Dataset is TKaDaoTable)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable)
  And (FAutoOpen)
  And (NOT FInGetting) Then
      Begin
        if ItemIndex > -1 Then
           Begin
             FInGetting := True;
             TKaDaoTable(DataSource.Dataset).IndexName := Items.Strings[ItemIndex];
             FInGetting := False;
           End;
      End;
End;


Procedure TKADaoIndexesListBox.ActiveChange(Sender: TObject);
Var
  II  : Integer;
  Txt : String;
  T   : TStringList;
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.Dataset is TKaDaoTable)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable)
  And (DataSource.DataSet.Active) Then
      Begin
        T  := TStringList.Create;
        Try
         TKaDaoTable(DataSource.Dataset).GetIndexNames(T);
         Txt := T.Text;
        Finally
         T.Free;
        End;
        if Txt <> Items.Text Then Items.Text := Txt;
        II := Items.IndexOf(TKaDaoTable(DataSource.Dataset).IndexName);
        if II <> -1 Then
           Begin
             FInGetting := True;
             ItemIndex := II;
             FInGetting := False;
           End;
      End;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.Dataset is TKaDaoTable)
  And (NOT DataSource.DataSet.Active) Then
      Begin
        Items.Clear;
      End;
End;

Function TKADaoIndexesListBox.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoIndexesListBox.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

Procedure TKADaoIndexesListBox.SetAutoOpen(Value: Boolean);
Begin
 FAutoOpen := Value;
 Items.Clear;
 ActiveChange(Self);
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoIndexesListBox]);
end;

end.
