unit KA.Data.KADao.TablesCheckListBox;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.CheckLst,
  KA.Data.KADao.Table;

type
  TViewObjects = (Tables,QueryDefs,All);

type
  TKADaoTablesCheckListBox = class(TCheckListBox)
  private
    { Private declarations }
    FDataLink       : TFieldDataLink;
    FInGetting      : Boolean;
    FAutoOpen       : Boolean;
    FViewType       : TViewObjects;
    Procedure          ActiveChange(Sender: TObject);
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   SetViewType(Value : TViewObjects);
    Procedure   SetAutoOpen(Value: Boolean);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   Loaded; override;
    Procedure   Click; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
  published
    { Published declarations }
    Property    AutoOpen          : Boolean     Read FAutoOpen        Write SetAutoOpen;
    Property    DataSource        : TDataSource  Read GetDataSource Write SetDataSource;
    Property    ViewType          : TViewObjects Read FViewType       Write SetViewType;
  end;

procedure Register;

implementation
Const
  dbOpenTable   = 1;
  dbOpenDynaset = 2;

Constructor TKADaoTablesCheckListBox.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FInGetting                := False;
  FAutoOpen                 := True;
  FViewType                 := Tables;
  FDataLink                 := TFieldDataLink.Create;
  FDataLink.Control         := Self;
  FDataLink.OnActiveChange  := ActiveChange;
End;

Destructor TKADaoTablesCheckListBox.Destroy;
Begin
 FDataLink.Free;
 FDataLink := nil;
 inherited Destroy;
End;

procedure TKADaoTablesCheckListBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then
      Begin
        DataSource := nil;
      End;
end;


Procedure TKADaoTablesCheckListBox.Loaded;
Begin
 Inherited Loaded;
 Items.Clear;
End;

Procedure TKADaoTablesCheckListBox.Click;
Var
  TT : Integer;
Begin
  Inherited Click;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.Dataset is TKaDaoTable)
  And Assigned(TKaDaoTable(DataSource.Dataset).Database)
  And TKaDaoTable(DataSource.Dataset).Database.Connected
  And (FAutoOpen)
  And (NOT FInGetting) Then
      Begin
        if ItemIndex > -1 Then
           Begin
             TT := TKaDaoTable(DataSource.Dataset).TableType;
             FDataLink.DataSet.Close;
             Case FViewType of
                  Tables    :  Begin
                                 TKaDaoTable(DataSource.Dataset).TableType := TT;
                                 TKaDaoTable(DataSource.Dataset).TableName := Items.Strings[ItemIndex];
                               End;
                  QueryDefs :  Begin
                                 TKaDaoTable(DataSource.Dataset).TableType   := dbOpenDynaset;
                                 TKaDaoTable(DataSource.Dataset).QueryDefName := Items.Strings[ItemIndex];
                               End;
                  All       :  Begin
                                 if TKaDaoTable(DataSource.Dataset).Database.TableNames.IndexOf(Items.Strings[ItemIndex]) <> -1 Then
                                    Begin
                                      TKaDaoTable(DataSource.Dataset).TableType := TT;
                                      TKaDaoTable(DataSource.Dataset).TableName := Items.Strings[ItemIndex];
                                    End;
                                 if TKaDaoTable(DataSource.Dataset).Database.QueryDefNames.IndexOf(Items.Strings[ItemIndex]) <> -1 Then
                                    Begin
                                      TKaDaoTable(DataSource.Dataset).TableType    := dbOpenDynaset;
                                      TKaDaoTable(DataSource.Dataset).QueryDefName := Items.Strings[ItemIndex];
                                    End;
                               End;
             End;
             Try
               TKaDaoTable(DataSource.Dataset).ReadOnly:=False;
               if TKaDaoTable(DataSource.Dataset).QueryDefName <> '' Then
                   Begin
                     If  (TKaDaoTable(DataSource.Dataset).Database.CoreDatabase.QueryDefs.Item[TKaDaoTable(DataSource.Dataset).QueryDefName].Parameters.Count > 0)
                     And (TKaDaoTable(DataSource.Dataset).PromptQueryDefParameters) Then FDataLink.DataSet.Open
                     Else
                     If  (TKaDaoTable(DataSource.Dataset).Database.CoreDatabase.QueryDefs.Item[TKaDaoTable(DataSource.Dataset).QueryDefName].Parameters.Count = 0) Then FDataLink.DataSet.Open
                   End
               Else
                   Begin
                     FDataLink.DataSet.Open;
                   End;
             Except
             End;
           End;
      End;
End;


Procedure TKADaoTablesCheckListBox.ActiveChange(Sender: TObject);
Var
  II  : Integer;
  Txt : String;
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.Dataset is TKaDaoTable)
  And (DataSource.DataSet.Active) Then
      Begin
        II := -1;
        Case FViewType of
             Tables    : Txt := TKaDaoTable(DataSource.Dataset).Database.TableNames.Text;
             QueryDefs : Txt := TKaDaoTable(DataSource.Dataset).Database.QueryDefNames.Text;
             All       : Txt := TKaDaoTable(DataSource.Dataset).Database.TableNames.Text + TKaDaoTable(DataSource.Dataset).Database.QueryDefNames.Text;
        End;
        if Txt <> Items.Text Then Items.Text := Txt;
        Case FViewType of
             Tables    : II := Items.IndexOf(TKaDaoTable(DataSource.Dataset).TableName);
             QueryDefs : II := Items.IndexOf(TKaDaoTable(DataSource.Dataset).QueryDefName);
             All       : Begin
                          II := Items.IndexOf(TKaDaoTable(DataSource.Dataset).TableName);
                          if II = -1 Then II := Items.IndexOf(TKaDaoTable(DataSource.Dataset).QueryDefName);
                         End;
        End;
        if II <> -1 Then
           Begin
             FInGetting := True;
             ItemIndex := II;
             FInGetting := False;
           End;
      End;
End;


Procedure TKADaoTablesCheckListBox.SetViewType(Value : TViewObjects);
Begin
  FViewType   := Value;
  ActiveChange(Self);
End;

Function TKADaoTablesCheckListBox.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoTablesCheckListBox.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

Procedure TKADaoTablesCheckListBox.SetAutoOpen(Value: Boolean);
Begin
 FAutoOpen := Value;
 Items.Clear;
 ActiveChange(Self);
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoTablesCheckListBox]);
end;

end.
